@@ -31,21 +31,15 @@
  *
  *   - pure status flags, reported by the data layer, which must be cleared
  *     before doing further I/O :
- *     CF_*_NULL, CF_*_PARTIAL
+ *     CF_*_EVENT, CF_*_PARTIAL
  *
  *   - pure status flags, reported by stream connector layer, which must also
  *     be cleared before doing further I/O :
- *     CF_*_TIMEOUT, CF_*_ERROR
+ *     CF_*_TIMEOUT
  *
  *   - read-only indicators reported by lower data levels :
  *     CF_STREAMER, CF_STREAMER_FAST
  *
- *   - write-once status flags reported by the stream connector layer :
- *     CF_SHUTR, CF_SHUTW
- *
- *   - persistent control flags managed only by application level :
- *     CF_SHUT*_NOW, CF_*_ENA
- *
  * The flags have been arranged for readability, so that the read and write
  * bits have the same position in a byte (read being the lower byte and write
  * the second one). All flag names are relative to the channel. For instance,
@@ -53,80 +47,41 @@
  * Please also update the chn_show_flags() function below in case of changes.
  */
 
-#define CF_READ_NULL      0x00000001  /* last read detected on producer side */
-#define CF_READ_PARTIAL   0x00000002  /* some data were read from producer or a read exception occurred */
+#define CF_READ_EVENT     0x00000001  /* a read event detected on producer side */
+/* unused: 0x00000002 */
 #define CF_READ_TIMEOUT   0x00000004  /* timeout while waiting for producer */
-#define CF_READ_ERROR     0x00000008  /* unrecoverable error on producer side */
-#define CF_READ_ACTIVITY  (CF_READ_NULL|CF_READ_PARTIAL|CF_READ_ERROR)
+/* unused 0x00000008 */
 
-/* unused: 0x00000010 */
-#define CF_SHUTR          0x00000020  /* producer has already shut down */
-#define CF_SHUTR_NOW      0x00000040  /* the producer must shut down for reads ASAP */
-#define CF_READ_NOEXP     0x00000080  /* producer should not expire */
+/* unused: 0x00000010 - 0x00000080 */
 
-#define CF_WRITE_NULL     0x00000100  /* write(0) or connect() succeeded on consumer side */
-#define CF_WRITE_PARTIAL  0x00000200  /* some data were written to the consumer */
+#define CF_WRITE_EVENT    0x00000100  /* a write event detected on consumer side */
+/* unused: 0x00000200 */
 #define CF_WRITE_TIMEOUT  0x00000400  /* timeout while waiting for consumer */
-#define CF_WRITE_ERROR    0x00000800  /* unrecoverable error on consumer side */
-#define CF_WRITE_ACTIVITY (CF_WRITE_NULL|CF_WRITE_PARTIAL|CF_WRITE_ERROR)
+/* unused 0x00000800 */
 
 #define CF_WAKE_WRITE     0x00001000  /* wake the task up when there's write activity */
-#define CF_SHUTW          0x00002000  /* consumer has already shut down */
-#define CF_SHUTW_NOW      0x00004000  /* the consumer must shut down for writes ASAP */
+/* unused: 0x00002000 - 0x00004000 */
 #define CF_AUTO_CLOSE     0x00008000  /* producer can forward shutdown to other side */
 
-/* When CF_SHUTR_NOW is set, it is strictly forbidden for the producer to alter
- * the buffer contents. When CF_SHUTW_NOW is set, the consumer is free to perform
- * a shutw() when it has consumed the last contents, otherwise the session processor
- * will do it anyway.
- *
- * The SHUT* flags work like this :
- *
- *  SHUTR SHUTR_NOW  meaning
- *    0       0      normal case, connection still open and data is being read
- *    0       1      closing : the producer cannot feed data anymore but can close
- *    1       0      closed: the producer has closed its input channel.
- *    1       1      impossible
- *
- *  SHUTW SHUTW_NOW  meaning
- *    0       0      normal case, connection still open and data is being written
- *    0       1      closing: the consumer can send last data and may then close
- *    1       0      closed: the consumer has closed its output channel.
- *    1       1      impossible
- *
- * The SHUTW_NOW flag should be set by the session processor when SHUTR and AUTO_CLOSE
- * are both set. And it may also be set by the producer when it detects SHUTR while
- * directly forwarding data to the consumer.
- *
- * The SHUTR_NOW flag is mostly used to force the producer to abort when an error is
- * detected on the consumer side.
- */
-
 #define CF_STREAMER       0x00010000  /* the producer is identified as streaming data */
 #define CF_STREAMER_FAST  0x00020000  /* the consumer seems to eat the stream very fast */
 
 #define CF_WROTE_DATA     0x00040000  /* some data were sent from this buffer */
-#define CF_ANA_TIMEOUT    0x00080000  /* the analyser timeout has expired */
-#define CF_READ_ATTACHED  0x00100000  /* the read side is attached for the first time */
+/* unused 0x00080000 - 0x00100000  */
 #define CF_KERN_SPLICING  0x00200000  /* kernel splicing desired for this channel */
-#define CF_READ_DONTWAIT  0x00400000  /* wake the task up after every read (eg: HTTP request) */
+/* unused 0x00400000 */
 #define CF_AUTO_CONNECT   0x00800000  /* consumer may attempt to establish a new connection */
 
 #define CF_DONT_READ      0x01000000  /* disable reading for now */
-#define CF_EXPECT_MORE    0x02000000  /* more data expected to be sent very soon (one-shoot) */
-#define CF_SEND_DONTWAIT  0x04000000  /* don't wait for sending data (one-shoot) */
-#define CF_NEVER_WAIT     0x08000000  /* never wait for sending data (permanent) */
+/* unused 0x02000000 - 0x08000000 */
 
 #define CF_WAKE_ONCE      0x10000000  /* pretend there is activity on this channel (one-shoot) */
 #define CF_FLT_ANALYZE    0x20000000  /* at least one filter is still analyzing this channel */
-#define CF_EOI            0x40000000  /* end-of-input has been reached */
+/* unuse 0x40000000 */
 #define CF_ISRESP         0x80000000  /* 0 = request channel, 1 = response channel */
 
 /* Masks which define input events for stream analysers */
-#define CF_MASK_ANALYSER  (CF_READ_ATTACHED|CF_READ_ACTIVITY|CF_READ_TIMEOUT|CF_ANA_TIMEOUT|CF_WRITE_ACTIVITY|CF_WAKE_ONCE)
-
-/* Mask for static flags which cause analysers to be woken up when they change */
-#define CF_MASK_STATIC    (CF_SHUTR|CF_SHUTW|CF_SHUTR_NOW|CF_SHUTW_NOW)
+#define CF_MASK_ANALYSER  (CF_READ_EVENT|CF_READ_TIMEOUT|CF_WRITE_EVENT|CF_WAKE_ONCE)
 
 /* This function is used to report flags in debugging tools. Please reflect
  * below any single-bit flag addition above in the same order via the
@@ -138,15 +93,15 @@ static forceinline char *chn_show_flags(char *buf, size_t len, const char *delim
 	/* prologue */
 	_(0);
 	/* flags */
-	_(CF_READ_NULL, _(CF_READ_PARTIAL, _(CF_READ_TIMEOUT, _(CF_READ_ERROR,
-	_(CF_SHUTR, _(CF_SHUTR_NOW, _(CF_READ_NOEXP, _(CF_WRITE_NULL,
-	_(CF_WRITE_PARTIAL, _(CF_WRITE_TIMEOUT, _(CF_WRITE_ERROR,
-	_(CF_WAKE_WRITE, _(CF_SHUTW, _(CF_SHUTW_NOW, _(CF_AUTO_CLOSE,
-	_(CF_STREAMER, _(CF_STREAMER_FAST, _(CF_WROTE_DATA, _(CF_ANA_TIMEOUT,
-	_(CF_READ_ATTACHED, _(CF_KERN_SPLICING, _(CF_READ_DONTWAIT,
-	_(CF_AUTO_CONNECT, _(CF_DONT_READ, _(CF_EXPECT_MORE,
-	_(CF_SEND_DONTWAIT, _(CF_NEVER_WAIT, _(CF_WAKE_ONCE, _(CF_FLT_ANALYZE,
-	_(CF_EOI, _(CF_ISRESP)))))))))))))))))))))))))))))));
+	_(CF_READ_EVENT, _(CF_READ_TIMEOUT,
+	_(CF_WRITE_EVENT,
+	_(CF_WRITE_TIMEOUT,
+	_(CF_WAKE_WRITE, _(CF_AUTO_CLOSE,
+	_(CF_STREAMER, _(CF_STREAMER_FAST, _(CF_WROTE_DATA,
+	_(CF_KERN_SPLICING,
+	_(CF_AUTO_CONNECT, _(CF_DONT_READ,
+	_(CF_WAKE_ONCE, _(CF_FLT_ANALYZE,
+	_(CF_ISRESP)))))))))))))));
 	/* epilogue */
 	_(~0U);
 	return buf;
@@ -254,10 +209,6 @@ struct channel {
 	unsigned char xfer_large;       /* number of consecutive large xfers */
 	unsigned char xfer_small;       /* number of consecutive small xfers */
 	unsigned long long total;       /* total data read */
-	int rex;                        /* expiration date for a read, in ticks */
-	int wex;                        /* expiration date for a write or connect, in ticks */
-	int rto;                        /* read timeout, in ticks */
-	int wto;                        /* write timeout, in ticks */
 	int analyse_exp;                /* expiration date for current analysers (if set) */
 };
 
