@@ -34,7 +34,7 @@ enum qcs_type {
 #define QC_CF_CONN_FULL 0x00000008 /* no stream buffers available on connection */
 #define QC_CF_APP_SHUT  0x00000010 /* Application layer shutdown done. */
 #define QC_CF_ERR_CONN  0x00000020 /* fatal error reported by transport layer */
-#define QC_CF_WAIT_FOR_HS 0x00000040 /* QUIC handshake has been completed */
+#define QC_CF_WAIT_HS   0x00000040 /* MUX init before QUIC handshake completed (0-RTT) */
 
 struct qcc {
 	struct connection *conn;
@@ -64,6 +64,8 @@ struct qcc {
 
 	/* flow-control fields set by the peer which we must respect. */
 	struct {
+		uint64_t ms_uni; /* max sub-ID of uni stream allowed by the peer */
+
 		uint64_t md; /* connection flow control limit updated on MAX_DATA frames reception */
 		uint64_t msd_bidi_l; /* initial max-stream-data from peer on local bidi streams */
 		uint64_t msd_bidi_r; /* initial max-stream-data from peer on remote bidi streams */
@@ -189,7 +191,7 @@ struct qcc_app_ops {
 	int (*init)(struct qcc *qcc);
 	int (*attach)(struct qcs *qcs, void *conn_ctx);
 	ssize_t (*decode_qcs)(struct qcs *qcs, struct buffer *b, int fin);
-	size_t (*snd_buf)(struct qcs *qcs, struct htx *htx, size_t count);
+	size_t (*snd_buf)(struct qcs *qcs, struct htx *htx, size_t count, char *fin);
 	int (*close)(struct qcs *qcs, enum qcc_app_ops_close_side side);
 	void (*detach)(struct qcs *qcs);
 	int (*finalize)(void *ctx);
