@@ -32,7 +32,7 @@
 
 #include <haproxy/buf.h>
 #include <haproxy/chunk.h>
-#include <haproxy/ncbuf-t.h>
+#include <haproxy/ncbmbuf.h>
 #include <haproxy/net_helper.h>
 #include <haproxy/openssl-compat.h>
 #include <haproxy/ticks.h>
@@ -431,8 +431,8 @@ static inline void quic_path_init(struct quic_path *path, int ipv4,
 	quic_loss_init(&path->loss);
 	path->mtu = max_dgram_sz;
 	path->cwnd = QUIC_MIN(10 * max_dgram_sz, QUIC_MAX(max_dgram_sz << 1, 14720U));
-	path->mcwnd = path->cwnd;
-	path->min_cwnd = max_dgram_sz << 1;
+	path->cwnd_last_max = path->cwnd;
+	path->limit_min = max_dgram_sz << 1;
 	path->prep_in_flight = 0;
 	path->in_flight = 0;
 	path->ifae_pkts = 0;
