@@ -25,6 +25,7 @@
 #include <haproxy/obj_type-t.h>
 #include <haproxy/connection-t.h>
 #include <haproxy/show_flags-t.h>
+#include <haproxy/xref-t.h>
 
 /* Stream Endpoint Flags.
  * Please also update the se_show_flags() function below in case of changes.
@@ -67,17 +68,17 @@ enum se_flags {
 	SE_FL_MAY_SPLICE = 0x00040000,  /* The endpoint may use the kernel splicing to forward data to the other side (implies SE_FL_CAN_SPLICE) */
 	SE_FL_RCV_MORE   = 0x00080000,  /* Endpoint may have more bytes to transfer */
 	SE_FL_WANT_ROOM  = 0x00100000,  /* More bytes to transfer, but not enough room */
-	SE_FL_ENDP_MASK  = 0x001ff000,  /* Mask for flags set by the endpoint */
+	SE_FL_EXP_NO_DATA= 0x00200000,  /* No data expected by the endpoint */
+	SE_FL_ENDP_MASK  = 0x002ff000,  /* Mask for flags set by the endpoint */
 
 	/* following flags are supposed to be set by the app layer and read by
 	 * the endpoint :
 	 */
-	SE_FL_WAIT_FOR_HS   = 0x00200000,  /* This stream is waiting for handhskae */
-	SE_FL_KILL_CONN     = 0x00400000,  /* must kill the connection when the SC closes */
-	SE_FL_WAIT_DATA     = 0x00800000,  /* stream endpoint cannot work without more data from the stream's output */
-	SE_FL_WONT_CONSUME  = 0x01000000,  /* stream endpoint will not consume more data */
-	SE_FL_HAVE_NO_DATA  = 0x02000000,  /* the endpoint has no more data to deliver to the stream */
-	/* unused             0x04000000,*/
+	SE_FL_WAIT_FOR_HS   = 0x00400000,  /* This stream is waiting for handhskae */
+	SE_FL_KILL_CONN     = 0x00800000,  /* must kill the connection when the SC closes */
+	SE_FL_WAIT_DATA     = 0x01000000,  /* stream endpoint cannot work without more data from the stream's output */
+	SE_FL_WONT_CONSUME  = 0x02000000,  /* stream endpoint will not consume more data */
+	SE_FL_HAVE_NO_DATA  = 0x04000000,  /* the endpoint has no more data to deliver to the stream */
 	/* unused             0x08000000,*/
 	/* unused             0x10000000,*/
 	/* unused             0x20000000,*/
@@ -98,9 +99,9 @@ static forceinline char *se_show_flags(char *buf, size_t len, const char *delim,
 	_(SE_FL_SHRD, _(SE_FL_SHRR, _(SE_FL_SHWN, _(SE_FL_SHWS,
 	_(SE_FL_NOT_FIRST, _(SE_FL_WEBSOCKET, _(SE_FL_EOI, _(SE_FL_EOS,
 	_(SE_FL_ERROR, _(SE_FL_ERR_PENDING, _(SE_FL_MAY_SPLICE,
-	_(SE_FL_RCV_MORE, _(SE_FL_WANT_ROOM, _(SE_FL_WAIT_FOR_HS,
-	_(SE_FL_KILL_CONN, _(SE_FL_WAIT_DATA, _(SE_FL_WONT_CONSUME,
-	_(SE_FL_HAVE_NO_DATA, _(SE_FL_APPLET_NEED_CONN)))))))))))))))))))))));
+	_(SE_FL_RCV_MORE, _(SE_FL_WANT_ROOM, _(SE_FL_EXP_NO_DATA,
+	_(SE_FL_WAIT_FOR_HS, _(SE_FL_KILL_CONN, _(SE_FL_WAIT_DATA,
+	_(SE_FL_WONT_CONSUME, _(SE_FL_HAVE_NO_DATA, _(SE_FL_APPLET_NEED_CONN))))))))))))))))))))))));
 	/* epilogue */
 	_(~0U);
 	return buf;
@@ -109,13 +110,41 @@ static forceinline char *se_show_flags(char *buf, size_t len, const char *delim,
 
 /* stconn flags.
  * Please also update the sc_show_flags() function below in case of changes.
+ *
+ * When SC_FL_ABRT_WANTED/SC_FL_EOS is set, it is strictly forbidden for the
+ * producer to alter the buffer contents. In this case, the consumer is free to
+ * perform a shutdown when it has consumed the last contents, otherwise the
+ * session processor will do it anyway. SC_FL_ABRT* are set at the upper layer
+ * level (the stream) while SC_FL_EOS is set at the SE layer.
+ *
+ * The SC_FL_SHUT_WANTED flaga should be set by the session processor when
+ * SC_FLABRT_DONE/SC_FL_EOS and CF_AUTO_CLOSE are both set. And it may also be
+ * set by the producer when it detects SC_FL_EOS while directly forwarding data to the
+ * consumer.
+ *
+ * The SHUT/ABRT flags work like this :
+ *
+ *  ABRT_WANTED ABRT_DONE  meaning
+ *      0           0      normal case, connection still open and data is being read
+ *      1           0      closing : the producer cannot feed data anymore but can close
+ *     0/1          1      closed: the producer has closed its input channel.
+ *
+ *  SHUT_WANTED SHUT_DONE  meaning
+ *      0          0      normal case, connection still open and data is being written
+ *      1          0      closing: the consumer can send last data and may then close
+ *     0/1         1      closed: the consumer has closed its output channel.
+ *
+ *
+ * The ABRT_WANTED flag is mostly used to force the producer to abort when an error is
+ * detected on the consumer side.
+ *
  */
 enum sc_flags {
 	SC_FL_NONE          = 0x00000000,  /* Just for initialization purposes */
 	SC_FL_ISBACK        = 0x00000001,  /* Set for SC on back-side */
 
-	/* not used: 0x00000002 */
-	/* not used: 0x00000004 */
+	SC_FL_EOI           = 0x00000002,  /* End of input was reached. no more data will be received from the endpoint */
+	SC_FL_ERROR         = 0x00000004,  /* A fatal error was reported */
 
 	SC_FL_NOLINGER      = 0x00000008,  /* may close without lingering. One-shot. */
 	SC_FL_NOHALF        = 0x00000010,  /* no half close, close both sides at once */
@@ -125,6 +154,18 @@ enum sc_flags {
 	SC_FL_WONT_READ     = 0x00000080,  /* SC doesn't want to read data */
 	SC_FL_NEED_BUFF     = 0x00000100,  /* SC waits for an rx buffer allocation to complete */
 	SC_FL_NEED_ROOM     = 0x00000200,  /* SC needs more room in the rx buffer to store incoming data */
+
+	SC_FL_RCV_ONCE      = 0x00000400,  /* Don't loop to receive data. cleared after a successful receive */
+	SC_FL_SND_ASAP      = 0x00000800,  /* Don't wait for sending. cleared when all data were sent */
+	SC_FL_SND_NEVERWAIT = 0x00001000,  /* Never wait for sending (permanent) */
+	SC_FL_SND_EXP_MORE  = 0x00002000,  /* More data expected to be sent very soon. cleared when all data were sent */
+
+	SC_FL_ABRT_WANTED   = 0x00004000,  /* An abort was requested and must be performed ASAP (up side to down side) */
+	SC_FL_SHUT_WANTED   = 0x00008000,  /* A shutdown was requested and mux be performed ASAP (up side to down side) */
+	SC_FL_ABRT_DONE     = 0x00010000,  /* An abort was performed for the SC */
+	SC_FL_SHUT_DONE     = 0x00020000,  /* A shutdown was performed for the SC */
+
+	SC_FL_EOS           = 0x00040000,  /* End of stream was reached (from down side to up side) */
 };
 
 /* This function is used to report flags in debugging tools. Please reflect
@@ -137,9 +178,12 @@ static forceinline char *sc_show_flags(char *buf, size_t len, const char *delim,
 	/* prologue */
 	_(0);
 	/* flags */
-	_(SC_FL_ISBACK, _(SC_FL_NOLINGER, _(SC_FL_NOHALF,
+	_(SC_FL_ISBACK, _(SC_FL_EOI, _(SC_FL_ERROR, _(SC_FL_NOLINGER, _(SC_FL_NOHALF,
 	_(SC_FL_DONT_WAKE, _(SC_FL_INDEP_STR, _(SC_FL_WONT_READ,
-	_(SC_FL_NEED_BUFF, _(SC_FL_NEED_ROOM))))))));
+	_(SC_FL_NEED_BUFF, _(SC_FL_NEED_ROOM,
+        _(SC_FL_RCV_ONCE, _(SC_FL_SND_ASAP, _(SC_FL_SND_NEVERWAIT, _(SC_FL_SND_EXP_MORE,
+	_(SC_FL_ABRT_WANTED, _(SC_FL_SHUT_WANTED, _(SC_FL_ABRT_DONE, _(SC_FL_SHUT_DONE,
+	_(SC_FL_EOS)))))))))))))))))));
 	/* epilogue */
 	_(~0U);
 	return buf;
@@ -199,13 +243,26 @@ struct stconn;
  * <se>     is the stream endpoint, i.e. the mux stream or the appctx
  * <conn>   is the connection for connection-based streams
  * <sc>     is the stream connector we're attached to, or NULL
+ * <lra>    is the last read activity
+ * <fsb>    is the first send blocked
  * <flags>  SE_FL_*
-*/
+ * <xref>   cross reference with the opposite SC
+ *
+ * <lra> should be updated when a read activity is detected. It can be a
+ *       successful receive, when a shutr is reported or when receives are
+ *       unblocked.
+
+ * <fsb> should be updated when the first send of a series is blocked and reset
+ *       when a successful send is reported.
+ */
 struct sedesc {
 	void *se;
 	struct connection *conn;
 	struct stconn *sc;
 	unsigned int flags;
+	unsigned int lra;
+	unsigned int fsb;
+	struct xref xref;
 };
 
 /* sc_app_ops describes the application layer's operations and notification
@@ -215,8 +272,8 @@ struct sedesc {
 struct sc_app_ops {
 	void (*chk_rcv)(struct stconn *);    /* chk_rcv function, may not be null */
 	void (*chk_snd)(struct stconn *);    /* chk_snd function, may not be null */
-	void (*shutr)(struct stconn *);      /* shut read function, may not be null */
-	void (*shutw)(struct stconn *);      /* shut write function, may not be null */
+	void (*abort)(struct stconn *);      /* abort function, may not be null */
+	void (*shutdown)(struct stconn *);   /* shutdown function, may not be null */
 	int  (*wake)(struct stconn *);       /* data-layer callback to report activity */
 	char name[8];                        /* data layer name, zero-terminated */
 };
@@ -230,7 +287,11 @@ struct stconn {
 	/* 2 bytes hole here */
 
 	unsigned int flags;                  /* SC_FL_* */
-	unsigned int hcto;                   /* half-closed timeout (0 = unset) */
+	unsigned int ioto;                   /* I/O activity timeout */
+	ssize_t room_needed;                 /* free space in the input buffer required to receive more data.
+					      *    -1   : the SC is waiting for room but not on a specific amount of data
+					      *    >= 0 : min free space required to progress. 0 means SC must be unblocked ASAP
+					      */
 	struct wait_event wait_event;        /* We're in a wait list */
 	struct sedesc *sedesc;               /* points to the stream endpoint descriptor */
 	enum obj_type *app;                  /* points to the applicative point (stream or check) */
