@@ -144,9 +144,10 @@ struct h3c {
 
 DECLARE_STATIC_POOL(pool_head_h3c, "h3c", sizeof(struct h3c));
 
-#define H3_SF_UNI_INIT  0x00000001  /* stream type not parsed for unidirectional stream */
-#define H3_SF_UNI_NO_H3 0x00000002  /* unidirectional stream does not carry H3 frames */
-#define H3_SF_HAVE_CLEN 0x00000004  /* content-length header is present */
+#define H3_SF_UNI_INIT     0x00000001  /* stream type not parsed for unidirectional stream */
+#define H3_SF_UNI_NO_H3    0x00000002  /* unidirectional stream does not carry H3 frames */
+#define H3_SF_HAVE_CLEN    0x00000004  /* content-length header is present */
+#define H3_SF_SENT_INTERIM 0x00000008  /* last response sent is 1xx interim. Used on FE side only. */
 
 struct h3s {
 	struct h3c *h3c;
@@ -1603,6 +1604,15 @@ static int h3_resp_headers_send(struct qcs *qcs, struct htx *htx)
 			sl = htx_get_blk_ptr(htx, blk);
 			/* TODO should be on h3 layer */
 			status = sl->info.res.status;
+			if (status >= 100 && status < 200) {
+				TRACE_USER("handling interim HTX response", H3_EV_STRM_SEND, qcs->qcc->conn, qcs);
+				BUG_ON(conn_is_back(qcs->qcc->conn)); /* H3_SF_SENT_INTERIM is FE side only. */
+				h3s->flags |= H3_SF_SENT_INTERIM;
+			}
+			else {
+				TRACE_USER("handling final HTX response", H3_EV_STRM_SEND, qcs->qcc->conn, qcs);
+				h3s->flags &= ~H3_SF_SENT_INTERIM;
+			}
 		}
 		else if (type == HTX_BLK_HDR) {
 			if (unlikely(hdr >= sizeof(list) / sizeof(list[0]) - 1)) {
@@ -1632,13 +1642,20 @@ static int h3_resp_headers_send(struct qcs *qcs, struct htx *htx)
 		goto err;
 	}
 
-	/* Buffer allocated just now : must be enough for frame type + length as a max varint size */
-	BUG_ON(b_room(res) < 5);
+	/* Reserve space for frame type + length as a max varint size. */
+	if (unlikely(b_contig_space(res) < 5)) {
+		/* Most of the times, h3_resp_headers_send() is only called one
+		 * time per stream, so buffer will be empty. However, this
+		 * assumption is invalid when handling interim responses, so
+		 * it's important to check out buffer remaining space.
+		 */
+		goto err;
+	}
 
 	b_reset(&outbuf);
 	outbuf = b_make(b_tail(res), b_contig_space(res), 0, 0);
 	/* Start the headers after frame type + length */
-	headers_buf = b_make(b_head(res) + 5, b_size(res) - 5, 0, 0);
+	headers_buf = b_make(b_tail(res) + 5, b_contig_space(res) - 5, 0, 0);
 
 	if (qpack_encode_field_section_line(&headers_buf)) {
 		h3c->err = H3_INTERNAL_ERROR;
@@ -1688,10 +1705,23 @@ static int h3_resp_headers_send(struct qcs *qcs, struct htx *htx)
 	/* Now that all headers are encoded, we are certain that res buffer is
 	 * big enough
 	 */
-	frame_length_size = quic_int_getsize(b_data(&headers_buf));
-	res->head += 4 - frame_length_size;
-	b_putchr(res, 0x01); /* h3 HEADERS frame type */
-	b_quic_enc_int(res, b_data(&headers_buf), 0);
+
+	/* Encode H3 HEADERS frame type + frame length. If buffer is empty,
+	 * length is encoded on min varint size and buf head is realigned.
+	 * However, if it still contains a previously encoded interim response
+	 * not yet emitted, buf head must not be adjusted and frame length is
+	 * encoded as a 4-bytes varint so that responses are contiguous.
+	 */
+	if (!b_data(res)) {
+		frame_length_size = quic_int_getsize(b_data(&headers_buf));
+		res->head += 4 - frame_length_size;
+		b_putchr(res, 0x01); /* h3 HEADERS frame type */
+		b_quic_enc_int(res, b_data(&headers_buf), 0);
+	}
+	else {
+		b_putchr(res, 0x01); /* h3 HEADERS frame type */
+		b_quic_enc_int(res, b_data(&headers_buf), 4);
+	}
 	b_add(res, b_data(&headers_buf));
 
 	ret = 0;
@@ -1944,7 +1974,7 @@ static int h3_resp_data_send(struct qcs *qcs, struct htx *htx, size_t count)
 	return -1;
 }
 
-static size_t h3_snd_buf(struct qcs *qcs, struct htx *htx, size_t count)
+static size_t h3_snd_buf(struct qcs *qcs, struct htx *htx, size_t count, char *fin)
 {
 	struct h3s *h3s = qcs->ctx;
 	struct h3c *h3c = h3s->h3c;
@@ -1953,10 +1983,15 @@ static size_t h3_snd_buf(struct qcs *qcs, struct htx *htx, size_t count)
 	struct htx_blk *blk;
 	uint32_t bsize;
 	int32_t idx;
+	char eom;
 	int ret = 0;
 
 	TRACE_ENTER(H3_EV_STRM_SEND, qcs->qcc->conn, qcs);
 
+	*fin = 0;
+	/* EOM is saved here, useful if 0-copy is performed with HTX buf. */
+	eom = htx->flags & HTX_FL_EOM;
+
 	while (count && !htx_is_empty(htx) &&
 	       !(qcs->flags & QC_SF_BLK_MROOM) && !h3c->err) {
 		idx = htx_get_head(htx);
@@ -1969,7 +2004,7 @@ static size_t h3_snd_buf(struct qcs *qcs, struct htx *htx, size_t count)
 
 		switch (btype) {
 		case HTX_BLK_RES_SL:
-			/* start-line -> HEADERS h3 frame */
+			/* start-line -> HEADERS h3 frame (FE side) */
 			ret = h3_resp_headers_send(qcs, htx);
 			if (ret > 0) {
 				total += ret;
@@ -2020,6 +2055,11 @@ static size_t h3_snd_buf(struct qcs *qcs, struct htx *htx, size_t count)
 	}
 
  out:
+	if (eom && htx_is_empty(htx) && !(h3s->flags & H3_SF_SENT_INTERIM)) {
+		TRACE_USER("transcoding last HTX message", H3_EV_STRM_SEND, qcs->qcc->conn, qcs);
+		*fin = 1;
+	}
+
 	TRACE_LEAVE(H3_EV_STRM_SEND, qcs->qcc->conn, qcs);
 	return total;
 }
@@ -2144,6 +2184,19 @@ static int h3_finalize(void *ctx)
 	struct h3c *h3c = ctx;
 	struct qcs *qcs;
 
+	/* RFC 9114 6.2. Unidirectional Streams
+	 *
+	 * Each endpoint needs to create at least one unidirectional stream for
+	 * the HTTP control stream. QPACK requires two additional unidirectional
+	 * streams, and other extensions might require further streams.
+	 * Therefore, the transport parameters sent by both clients and servers
+	 * MUST allow the peer to create at least three unidirectional streams.
+	 */
+	if (qcc_fctl_avail_streams(h3c->qcc, 0) < 3) {
+		qcc_set_error(h3c->qcc, H3_GENERAL_PROTOCOL_ERROR, 1);
+		return 1;
+	}
+
 	qcs = qcc_init_stream_local(h3c->qcc, 0);
 	if (!qcs)
 		return 1;
