@@ -34,7 +34,7 @@
 THREAD_LOCAL size_t pool_cache_bytes = 0;                /* total cache size */
 THREAD_LOCAL size_t pool_cache_count = 0;                /* #cache objects   */
 
-static struct list pools __read_mostly = LIST_HEAD_INIT(pools);
+struct list pools __read_mostly = LIST_HEAD_INIT(pools);
 int mem_poison_byte __read_mostly = 'P';
 uint pool_debugging __read_mostly =               /* set of POOL_DBG_* flags */
 #ifdef DEBUG_FAIL_ALLOC
@@ -856,6 +856,9 @@ void *pool_destroy(struct pool_head *pool)
 			/* note that if used == 0, the cache is empty */
 			free(pool->base_addr);
 		}
+
+		/* make sure this pool is no longer referenced in memory profiling */
+		memprof_remove_stale_info(pool);
 	}
 	return NULL;
 }
