@@ -56,7 +56,7 @@ static void quic_cc_nr_slow_start(struct quic_cc *cc)
 	struct nr *nr = quic_cc_priv(cc);
 
 	path = container_of(cc, struct quic_path, cc);
-	path->cwnd = path->min_cwnd;
+	quic_cc_path_reset(path);
 	/* Re-entering slow start state. */
 	nr->state = QUIC_CC_ST_SS;
 	/* Recovery start time reset */
@@ -72,7 +72,7 @@ static void quic_cc_nr_enter_recovery(struct quic_cc *cc)
 	path = container_of(cc, struct quic_path, cc);
 	nr->recovery_start_time = now_ms;
 	nr->ssthresh = path->cwnd >> 1;
-	path->cwnd = QUIC_MAX(nr->ssthresh, (uint32_t)path->min_cwnd);
+	quic_cc_path_set(path, nr->ssthresh);
 	nr->state = QUIC_CC_ST_RP;
 }
 
@@ -87,10 +87,7 @@ static void quic_cc_nr_ss_cb(struct quic_cc *cc, struct quic_cc_event *ev)
 	path = container_of(cc, struct quic_path, cc);
 	switch (ev->type) {
 	case QUIC_CC_EVT_ACK:
-		if (quic_cwnd_may_increase(path)) {
-			path->cwnd += ev->ack.acked;
-			path->mcwnd = QUIC_MAX(path->cwnd, path->mcwnd);
-		}
+		quic_cc_path_inc(path, ev->ack.acked);
 		/* Exit to congestion avoidance if slow start threshold is reached. */
 		if (path->cwnd > nr->ssthresh)
 			nr->state = QUIC_CC_ST_CA;
@@ -126,10 +123,7 @@ static void quic_cc_nr_ca_cb(struct quic_cc *cc, struct quic_cc_event *ev)
 		 */
 		acked = ev->ack.acked * path->mtu + nr->remain_acked;
 		nr->remain_acked = acked % path->cwnd;
-		if (quic_cwnd_may_increase(path)) {
-			path->cwnd += acked / path->cwnd;
-			path->mcwnd = QUIC_MAX(path->cwnd, path->mcwnd);
-		}
+		quic_cc_path_inc(path, acked / path->cwnd);
 		break;
 	}
 
@@ -169,7 +163,7 @@ static void quic_cc_nr_rp_cb(struct quic_cc *cc, struct quic_cc_event *ev)
 
 		nr->state = QUIC_CC_ST_CA;
 		nr->recovery_start_time = TICK_ETERNITY;
-		path->cwnd = nr->ssthresh;
+		quic_cc_path_set(path, nr->ssthresh);
 		break;
 	case QUIC_CC_EVT_LOSS:
 		/* Do nothing */
@@ -189,10 +183,10 @@ static void quic_cc_nr_state_trace(struct buffer *buf, const struct quic_cc *cc)
 	struct nr *nr = quic_cc_priv(cc);
 
 	path = container_of(cc, struct quic_path, cc);
-	chunk_appendf(buf, " state=%s cwnd=%llu mcwnd=%llu ssthresh=%ld rpst=%dms pktloss=%llu",
+	chunk_appendf(buf, " state=%s cwnd=%llu cwnd_last_max=%llu ssthresh=%ld rpst=%dms pktloss=%llu",
 	              quic_cc_state_str(nr->state),
 	              (unsigned long long)path->cwnd,
-	              (unsigned long long)path->mcwnd,
+	              (unsigned long long)path->cwnd_last_max,
 	              (long)nr->ssthresh,
 	              !tick_isset(nr->recovery_start_time) ? -1 :
 	              TICKS_TO_MS(tick_remain(nr->recovery_start_time, now_ms)),
