@@ -1754,6 +1754,8 @@ void sc_conn_sync_send(struct stconn *sc)
 		return;
 
 	sc_conn_send(sc);
+	if (oc->flags & CF_WRITE_EVENT)
+		oc->flags |= CF_WAKE_ONCE;
 }
 
 /* Called by I/O handlers after completion.. It propagates
